Enum socket2::InterfaceIndexOrAddress [−][src]
A local interface specified by its index or an address assigned to it.
Index(0) and Address(Ipv4Addr::UNSPECIFIED) are equivalent and indicate
that an appropriate interface should be selected by the system.
Variants
Index(u32)
Tuple Fields
0: u32An interface index.
Address(Ipv4Addr)
Tuple Fields
0: Ipv4AddrAn address assigned to an interface.