Enum InterfaceIndexOrAddress

enum InterfaceIndexOrAddress

A local interface specified by its index or an address assigned to it.

Index(0) and Address(Ipv4Addr::UNSPECIFIED) are equivalent and indicate that an appropriate interface should be selected by the system.

Variants

Index(u32)

An interface index.

Address(std::net::Ipv4Addr)

An address assigned to an interface.

Implementations

impl Clone for InterfaceIndexOrAddress

fn clone(self: &Self) -> InterfaceIndexOrAddress

impl Copy for InterfaceIndexOrAddress

impl Debug for InterfaceIndexOrAddress

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Freeze for InterfaceIndexOrAddress

impl RefUnwindSafe for InterfaceIndexOrAddress

impl Send for InterfaceIndexOrAddress

impl Sync for InterfaceIndexOrAddress

impl Unpin for InterfaceIndexOrAddress

impl UnsafeUnpin for InterfaceIndexOrAddress

impl UnwindSafe for InterfaceIndexOrAddress

impl<T> Any for InterfaceIndexOrAddress

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for InterfaceIndexOrAddress

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for InterfaceIndexOrAddress

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for InterfaceIndexOrAddress

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for InterfaceIndexOrAddress

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> ToOwned for InterfaceIndexOrAddress

fn to_owned(self: &Self) -> T
fn clone_into(self: &Self, target: &mut T)

impl<T, U> Into for InterfaceIndexOrAddress

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for InterfaceIndexOrAddress

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for InterfaceIndexOrAddress

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>